# functions to classify bird names to latin species names and countries to continents
# S. J. Lycett
# 14 Dec 2011 - extract from from matrixTranslate.R
# 3 April 2012 - added great cormorant to pel, and Bulgaria to Europe
# 23 May 2012 - added more birds
# 24 May 2012 - added more countries
# 13 July 2012 - changed egret from ans to cic

######################################################################
# function to classify birds in H5N1 - to bird order

# Bird Orders

# Acc	Accipitriformes	http://en.wikipedia.org/wiki/Accipitriformes	Eagles, Hawks (falcons are separate)
# Ans	Anseriformes	http://tolweb.org/Anseriformes	Ducks, Geese, Swans
# Cha	Charadriiformes	http://tolweb.org/Charadriiformes	Shorebirds
# Cic	Ciconiiformes	http://tolweb.org/Ciconiiformes/26332	Storks, Herons
# Cor Coraciiformes	http://en.wikipedia.org/wiki/Coraciiformes - like pas.
# Col Columbiformes	http://en.wikipedia.org/wiki/Columbiformes, doves and pigeons
# Gal	Galliformes	http://tolweb.org/Galliformes	Fowl, Quail
# Gru	Gruiformes	http://en.wikipedia.org/wiki/Gruiformes	Coot, Moorhen
# Fal	Falconiformes	http://tolweb.org/Falconiformes/26379	Falcons
# Pas	Passeriformes	http://tolweb.org/Passeriformes	Perching birds (e.g Sparrows)
# Pel Pelecaniforms	http://en.wikipedia.org/wiki/Pelecaniformes Pelican
# Pic Piciformes		http://en.wikipedia.org/wiki/Piciformes - like pas, includes woodpeckers
# Pro	Procellariiformes	http://en.wikipedia.org/wiki/Procellariiformes	Shearwater
# Psi Psittaciformes	http://en.wikipedia.org/wiki/Parrot	Parrot
# Rhe	Rheiformes	http://en.wikipedia.org/wiki/Rhea_(bird)	Rhea (looks like an emu)
# Str	Struthioniformes	http://en.wikipedia.org/wiki/Ostrich	Ostrich, Emu
# Tin Tinamiformes	http://en.wikipedia.org/wiki/Red-winged_Tinamou Tinamou (Brazil)

# birdNames is a vector

birdClass	<- function( birdNames ) {

	birdNames	<- tolower(birdNames)

	# nasty coding of ck for chicken messes up duck !!
	inds		<- which((birdNames=="ck") | (birdNames=="chick"))
	if (length(inds) > 0) {
		birdNames[inds] <- "chicken"
	}	

	# crowned gets synoned with crow - so remove all crowneds
	inds		<- grep("crowned", birdNames)
	if (length(inds) > 0) {
		for (i in 1:length(inds)) {
			birdNames[inds[i]] = sub("crowned", "", birdNames[inds[i]])
		}
	}

	birdOrders	<- array("-", length(birdNames))

	acc	<- c("eagle","hawk","buzzard","bussard","hooded vulture")

	ans	<- c("dk","duck","mallard","teal","goose","gs","swan","cygnus","wigeon",
			"munia","grebe","goldeneye","pochard","goosander",
			"waterfowl","pintail","gadwall","garganey","platyrhynchos","crecca",
			"plathyrhynchos","shoveler","aquatic bird","aquaticbird",
			"widgeon","bufflehead","eider","scoter","redhead","water fowl","watercock","anas acuta",
			"greater scaup","mergus albellus","smew")

	cic	<- c("stork","heron","egret")

	cor	<- c("rollers")

	col	<- c("peaceful dove")

	cha	<- c("shorebird","gull","tern","turnstone","dunlin","sandpiper","sanderling",
			"red knot","redknot","knot","plover","red-necked stint","redneckedstint","curlew")

	gal	<- c("chicken","pheasant","quail","turkey","sck","guinea",
			"chukar","bustard","peacock","peahen","peafowl","partridge","ph","qa",
			"poultry","chukkar","chukka")

	gru	<- c("coot","moorhen","hooded crane","black-neck crane")

	fal	<- c("falcon","kestrel","peregrine","harrier","owl")

	pas	<- c("sparrow","crow","magpie","pigeon","blackbird","myna",
			"starling","wild bird","wildbird","shrike","robin","softbill",
			"japanese white eye","japanesewhiteeye","common iora","commoniora",
			"fairy bluebird","fairybluebird","rook",
			"babbler","black bulbul","golden mountain thrush",
			"japanese white-eye","silver-eared mesia")

	psi	<- c("parrot","conure","parakeet","psittacine","macaw")

	pro	<- c("shearwater")	
	
	rhe	<- c("rhea")

	str	<- c("ostrich","emu")

	tin	<- c("red-winged tinamou", "redwingedtinamou")

	pel	<- c("pelican","great cormorant","greatcormorant", "cormorant", "ardea cinerea", "grey heron")

	pic	<- c("great barbet")

	orders <- list(	acc=acc,ans=ans,cic=cic,cor=cor,col=col,
				cha=cha,gru=gru,gal=gal,fal=fal,
				pas=pas,pel=pel,pro=pro,pic=pic,rhe=rhe,str=str,psi=psi,tin=tin)

	for (i in 1:length(orders)) {
		oo	<- unlist(orders[i])
		for (j in 1:length(oo)) {
			inds	<- grep(oo[j], birdNames)
			if (length(inds) > 0) {
				birdOrders[inds] <- attributes(orders[i])$names
			}
		}
	}	

	inds	<- grep("feces", birdNames)
	if (length(inds) > 0) {
		birdOrders[inds] <- "-"
	}

	return( birdOrders )

}

#####################################################################################################
# function to classify countries to continents

# check this before use - it only has limited values
# updated 16 Nov 2011
# updated 31 Jan 2012
# updated 14 Feb 2012
# updated 17 Feb 2012
# updated 24 May 2012
# updated 9  Jul 2012
continentClass	<- function( countries ) {
	Europe		<- c(	"UK","France","Austria","Belgium","Bolivia","CzechRepublic","Denmark","Estonia",
					"Germany","Greece","Italy","Luxembourg","Netherlands",
					"Norway","Poland","Turkey","Spain","Serbia",
					"UnitedKingdom","Ireland","Hungary","Sweden","Portugal",
					"Slovenia","Croatia","Herzegovina", "Bulgaria",
					"Switzerland","BosniaandHerzegovina","Slovakia")

	NorthAmerica	<- c("USA","Canada","Mexico")
	CentralAmerica	<- c("ElSalvador","Nicaragua","Haiti","DominicanRepublic",
					"Guatemala","Panama","PuertoRico")

	SouthAmerica	<- c("Chile","Colombia","Argentina","Ecuador","Peru","Brazil")

	Africa		<- c("Ethiopia","Nigeria","Mali","Senegal",
					"Africa","Sudan","Egypt","BurkinaFaso", "Niger", 
					"IvoryCoast", "CotedIvoire", "Zambia", "SouthAfrica","Ghana")

	Asia			<- c("India","Malaysia","HongKong","China","Singapore","SouthKorea","Thailand","Japan",
					"Cambodia","Taiwan","Myanmar","VietNam","Pakistan",
					"Bangladesh","Bhutan", "Indonesia", "Laos","Korea","Malaya")

	Oceania		<- c("Australia","NewZealand","Guam","NewCaledonia")

	MiddleEast		<- c("Jordan","Israel","Afghanistan","Arabia",
					"Kuwait","GazaStrip", "Iran", "Iraq", "SaudiArabia", "Lebanon",
					"UnitedArabEmirates","Qatar")

	CentralAsia		<- c("Kyrgyzstan","Turkmenistan","Russia","Mongolia","Kazakhstan","Ukraine","Uzbekistan")

	continents		<- array("-", length(countries))

	conts			<- list( 	Europe=Europe, NorthAmerica=NorthAmerica,
						SouthAmerica=c(CentralAmerica,SouthAmerica),
						Africa=Africa, Oceania=Oceania, Asia=c(Asia,CentralAsia,MiddleEast) )

	for (i in 1:length(conts)) {
		oo 	<- unlist(conts[i])
		
		for (j in 1:length(oo)) {
			inds 	<- which(countries==oo[j])
			if (length(inds) > 0) {
				continents[inds] <- attributes(conts[i])$names
			}	 
		}
	}
	
	return (continents)
}

